package ca.mcgill.sable.abc.impact.visualizer.model;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.swt.graphics.Image;

/**
 * The tree should be constructed from top to bottom
 * If nodes are instanceof AbstractTreeNode, the parent/childern relationship
 * is maintained automatically when creating a new node and removing children
 * @author Dehua Zhang
 */
public abstract class AbstractTreeNode implements ITreeNode {

	private ITreeNode parent;
	protected String name;
	protected Image image;
	private List<ITreeNode> children;
	
	// prevent non-argument constructor
	private AbstractTreeNode() {}

	private void addThisTo(ITreeNode parent) {
		if (parent instanceof AbstractTreeNode) {
			((AbstractTreeNode)parent).children.add(this);
		}
	}

	/**
	 * For extension purpose only
	 * child class should set name in the constructor
	 * if parent is instance of AbstractTreeNode, this is added to its children list
	 * @param parent
	 */
	protected AbstractTreeNode(ITreeNode parent) {
		addThisTo(parent);
		this.parent = parent;
		this.children = new ArrayList<ITreeNode>();
	}
	
//	/**
//	 * For extension purpose only
//	 * if parent is instance of AbstractTreeNode, this is added to its children list
//	 * @param parent
//	 * @param children
//	 */
//	protected AbstractTreeNode(ITreeNode parent, List<ITreeNode> children) {
//		this.parent = parent;
//		if (children == null) this.children = new ArrayList<ITreeNode>();
//		else this.children = new ArrayList<ITreeNode>(children);
//	}
	
	/**
	 * if parent is instance of AbstractTreeNode, this is added to its children list
	 * @param parent
	 * @param name
	 */
	public AbstractTreeNode(ITreeNode parent, String name, Image image) {
		addThisTo(parent);
		this.parent = parent;
		this.name = name;
		this.image = image;
		this.children = new ArrayList<ITreeNode>();
	}
	
//	/**
//	 * if parent is instance of AbstractTreeNode, this is added to its children list
//	 * @param parent
//	 * @param name
//	 * @param children
//	 */
//	public AbstractTreeNode(ITreeNode parent, String name, List<ITreeNode> children) {
//		this.parent = parent;
//		this.name = name;
//		if (children == null) this.children = new ArrayList<ITreeNode>();
//		else this.children = new ArrayList<ITreeNode>(children);
//	}
	
	public List<ITreeNode> getChildren() {
		return Collections.unmodifiableList(this.children);
	}
	
	public boolean hasChildren() {
		return !this.children.isEmpty();
	}

	/**
	 * if child is instance of AbstractTreeNode, its parent is set to null
	 * @param child
	 */
	public void removeChild(ITreeNode child) {
		if (child instanceof AbstractTreeNode) {
			((AbstractTreeNode)child).parent = null;
		}
		this.children.remove(child);
	}

	/**
	 * for childern that are instance of AbstractTreeNode, their parent is set to null
	 * @param children
	 */
	public void removeChildren(List<ITreeNode> children) {
		for (ITreeNode child : children) {
			if (child instanceof AbstractTreeNode) {
				((AbstractTreeNode)child).parent = null;
			}
		}
		this.children.removeAll(children);
	}

	public String getName() {
		return name;
	}
	
	public Image getImage() {
		return image;
	}
	
	public ITreeNode getParent() {
		return parent;
	}

	public String toString() {
		return getName();
	}
}
